@@ -211,7 +211,7 @@ decode_packet (GIOChannel           *source,
         ss_len = sizeof (clnt_ss);
         res = XdmcpFill (widget->priv->socket_fd, &buf, (XdmcpNetaddr)&clnt_ss, &ss_len);
         if G_UNLIKELY (! res) {
-                g_debug (_("XMCP: Could not create XDMCP buffer!"));
+                g_debug (_("XDMCP: Could not create XDMCP buffer!"));
                 return TRUE;
         }
 
