@@ -35,6 +35,7 @@
 
 #include "gdm-greeter-client.h"
 #include "gdm-marshal.h"
+#include "gdm-profile.h"
 
 #define GDM_GREETER_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_CLIENT, GdmGreeterClientPrivate))
 
@@ -66,6 +67,7 @@ enum {
         RESET,
         SELECTED_USER_CHANGED,
         DEFAULT_LANGUAGE_NAME_CHANGED,
+        DEFAULT_LAYOUT_NAME_CHANGED,
         DEFAULT_SESSION_NAME_CHANGED,
         TIMED_LOGIN_REQUESTED,
         USER_AUTHORIZED,
@@ -173,6 +175,13 @@ on_default_language_name_changed (GdmGreeterClient *client,
         emit_string_signal_for_message (client, "DefaultLanguageNameChanged", message, DEFAULT_LANGUAGE_NAME_CHANGED);
 }
 
+static void
+on_default_layout_name_changed (GdmGreeterClient *client,
+                                DBusMessage      *message)
+{
+        emit_string_signal_for_message (client, "DefaultLayoutNameChanged", message, DEFAULT_LAYOUT_NAME_CHANGED);
+}
+
 static void
 on_default_session_name_changed (GdmGreeterClient *client,
                                  DBusMessage      *message)
@@ -293,7 +302,9 @@ send_dbus_string_method (DBusConnection *connection,
                            error.message);
                 return FALSE;
         }
-        dbus_message_unref (reply);
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
         dbus_connection_flush (connection);
 
         return TRUE;
@@ -339,7 +350,9 @@ send_dbus_bool_method (DBusConnection *connection,
                            error.message);
                 return FALSE;
         }
-        dbus_message_unref (reply);
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
         dbus_connection_flush (connection);
 
         return TRUE;
@@ -378,7 +391,9 @@ send_dbus_void_method (DBusConnection *connection,
                            error.message);
                 return FALSE;
         }
-        dbus_message_unref (reply);
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
         dbus_connection_flush (connection);
 
         return TRUE;
@@ -444,6 +459,14 @@ gdm_greeter_client_call_select_language (GdmGreeterClient *client,
                                  text);
 }
 
+void
+gdm_greeter_client_call_select_layout (GdmGreeterClient *client,
+                                       const char       *text)
+{
+        send_dbus_string_method (client->priv->connection,
+                                 "SelectLayout",
+                                 text);
+}
 void
 gdm_greeter_client_call_select_user (GdmGreeterClient *client,
                                      const char       *text)
@@ -626,6 +649,8 @@ client_dbus_handle_message (DBusConnection *connection,
                 on_selected_user_changed (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "DefaultLanguageNameChanged")) {
                 on_default_language_name_changed (client, message);
+        } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "DefaultLayoutNameChanged")) {
+                on_default_layout_name_changed (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "DefaultSessionNameChanged")) {
                 on_default_session_name_changed (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "TimedLoginRequested")) {
@@ -682,6 +707,8 @@ gdm_greeter_client_start (GdmGreeterClient *client,
 
         g_return_val_if_fail (GDM_IS_GREETER_CLIENT (client), FALSE);
 
+        gdm_profile_start (NULL);
+
         ret = FALSE;
 
         if (client->priv->address == NULL) {
@@ -703,7 +730,7 @@ gdm_greeter_client_start (GdmGreeterClient *client,
                         g_set_error (error,
                                      GDM_GREETER_CLIENT_ERROR,
                                      GDM_GREETER_CLIENT_ERROR_GENERIC,
-                                     local_error.message);
+                                     "%s", local_error.message);
                         dbus_error_free (&local_error);
                 } else {
                         g_warning ("Unable to open connection");
@@ -724,6 +751,8 @@ gdm_greeter_client_start (GdmGreeterClient *client,
         ret = TRUE;
 
  out:
+        gdm_profile_end (NULL);
+
         return ret;
 }
 
@@ -884,6 +913,16 @@ gdm_greeter_client_class_init (GdmGreeterClientClass *klass)
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
+        gdm_greeter_client_signals[DEFAULT_LAYOUT_NAME_CHANGED] =
+                g_signal_new ("default-layout-name-changed",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterClientClass, default_layout_name_changed),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1, G_TYPE_STRING);
         gdm_greeter_client_signals[DEFAULT_SESSION_NAME_CHANGED] =
                 g_signal_new ("default-session-name-changed",
                               G_OBJECT_CLASS_TYPE (object_class),
