@@ -56,10 +56,6 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
-#ifdef HAVE_POLKIT_GNOME
-#include <polkit-gnome/polkit-gnome.h>
-#endif
-
 #include "gdm-settings-client.h"
 #include "gdm-settings-keys.h"
 #include "gdm-profile.h"
@@ -92,6 +88,7 @@
 #define KEY_GREETER_DIR             "/apps/gdm/simple-greeter"
 #define KEY_BANNER_MESSAGE_ENABLED  KEY_GREETER_DIR "/banner_message_enable"
 #define KEY_BANNER_MESSAGE_TEXT     KEY_GREETER_DIR "/banner_message_text"
+#define KEY_BANNER_MESSAGE_TEXT_NOCHOOSER     KEY_GREETER_DIR "/banner_message_text_nochooser"
 #define KEY_LOGO                    KEY_GREETER_DIR "/logo_icon_name"
 #define KEY_DISABLE_RESTART_BUTTONS KEY_GREETER_DIR "/disable_restart_buttons"
 #define GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_LOGIN_WINDOW, GdmGreeterLoginWindowPrivate))
@@ -103,6 +100,7 @@ enum {
 
 enum {
         LOGIN_BUTTON_HIDDEN = 0,
+        LOGIN_BUTTON_START_OTHER,
         LOGIN_BUTTON_ANSWER_QUERY,
         LOGIN_BUTTON_TIMED_LOGIN
 };
@@ -114,6 +112,7 @@ struct GdmGreeterLoginWindowPrivate
         GtkWidget       *auth_banner_label;
         guint            display_is_local : 1;
         guint            is_interactive : 1;
+        guint            user_chooser_loaded : 1;
         GConfClient     *client;
 
         gboolean         banner_message_enabled;
@@ -160,6 +159,10 @@ static void     restart_timed_login_timeout (GdmGreeterLoginWindow *login_window
 static void     on_user_unchosen            (GdmUserChooserWidget *user_chooser,
                                              GdmGreeterLoginWindow *login_window);
 
+static void     switch_mode                 (GdmGreeterLoginWindow *login_window,
+                                             int                    number);
+static void     update_banner_message       (GdmGreeterLoginWindow *login_window);
+
 G_DEFINE_TYPE (GdmGreeterLoginWindow, gdm_greeter_login_window, GTK_TYPE_WINDOW)
 
 static void
@@ -407,6 +410,17 @@ on_login_button_clicked_timed_login (GtkButton             *button,
 
         _gdm_greeter_login_window_set_interactive (login_window, TRUE);
 }
+static void
+on_login_button_clicked_start_other (GtkButton             *button,
+                                     GdmGreeterLoginWindow *login_window)
+{
+        g_debug ("GdmGreeterLoginWindow: starting OTHER login");
+
+        g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
+                       0, GDM_USER_CHOOSER_USER_OTHER);
+        g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
+        switch_mode (login_window, MODE_AUTHENTICATION);
+}
 
 static void
 set_log_in_button_mode (GdmGreeterLoginWindow *login_window,
@@ -427,6 +441,10 @@ set_log_in_button_mode (GdmGreeterLoginWindow *login_window,
         case LOGIN_BUTTON_HIDDEN:
                 gtk_widget_hide (button);
                 break;
+        case LOGIN_BUTTON_START_OTHER:
+                login_window->priv->login_button_handler_id = g_signal_connect (button, "clicked", G_CALLBACK (on_login_button_clicked_start_other), login_window);
+                gtk_widget_show (button);
+                break;
         case LOGIN_BUTTON_ANSWER_QUERY:
                 login_window->priv->login_button_handler_id = g_signal_connect (button, "clicked", G_CALLBACK (on_login_button_clicked_answer_query), login_window);
                 gtk_widget_show (button);
@@ -441,6 +459,24 @@ set_log_in_button_mode (GdmGreeterLoginWindow *login_window,
         }
 }
 
+static void
+adjust_other_login_visibility(GdmGreeterLoginWindow *login_window)
+{
+        if (! login_window->priv->user_chooser_loaded) {
+                return;
+        }
+
+        if (login_window->priv->dialog_mode != MODE_SELECTION) {
+                return;
+        }
+
+        if (gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) == 0) {
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_START_OTHER);
+        } else {
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
+        }
+}
+
 static gboolean
 can_suspend (GdmGreeterLoginWindow *login_window)
 {
@@ -543,7 +579,6 @@ switch_mode (GdmGreeterLoginWindow *login_window,
 
         switch (number) {
         case MODE_SELECTION:
-                gtk_widget_set_size_request (GTK_WIDGET (login_window), -1, -1);
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
 
                 show_widget (login_window, "cancel-button", FALSE);
@@ -568,9 +603,6 @@ switch_mode (GdmGreeterLoginWindow *login_window,
                 default_name = NULL;
                 break;
         case MODE_AUTHENTICATION:
-                gtk_widget_set_size_request (GTK_WIDGET (login_window),
-                                             GTK_WIDGET (login_window)->allocation.width,
-                                             -1);
                 show_widget (login_window, "cancel-button", TRUE);
                 show_widget (login_window, "shutdown-button", FALSE);
                 show_widget (login_window, "restart-button", FALSE);
@@ -712,7 +744,11 @@ reset_dialog (GdmGreeterLoginWindow *login_window)
         set_sensitive (login_window, TRUE);
         set_ready (login_window);
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
-        gdm_chooser_widget_propagate_pending_key_events (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+        update_banner_message (login_window);
+        adjust_other_login_visibility (login_window);
+        if (gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) >= 1) {
+                gdm_chooser_widget_propagate_pending_key_events (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+        }
 }
 
 static void
@@ -1034,119 +1070,6 @@ try_system_restart (DBusGConnection *connection,
         return res;
 }
 
-#ifdef HAVE_POLKIT_GNOME
-static void
-system_restart_auth_cb (PolKitAction          *action,
-                        gboolean               gained_privilege,
-                        GError                *error,
-                        GdmGreeterLoginWindow *login_window)
-{
-        GError          *local_error;
-        DBusGConnection *connection;
-        gboolean         res;
-
-        g_debug ("GdmGreeterLoginWindow: system restart auth callback gained=%s", gained_privilege ? "yes" : "no");
-
-        if (! gained_privilege) {
-                if (error != NULL) {
-                        g_warning ("GdmGreeterLoginWindow: system restart error: %s", error->message);
-                }
-                return;
-        }
-
-        local_error = NULL;
-        connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &local_error);
-        if (connection == NULL) {
-                g_warning ("Unable to get system bus connection: %s", local_error->message);
-                g_error_free (local_error);
-                return;
-        }
-
-        res = try_system_restart (connection, &local_error);
-        if (! res) {
-                g_warning ("Unable to restart system: %s", local_error->message);
-                g_error_free (local_error);
-                return;
-        }
-}
-
-static void
-system_stop_auth_cb (PolKitAction          *action,
-                     gboolean               gained_privilege,
-                     GError                *error,
-                     GdmGreeterLoginWindow *login_window)
-{
-        GError          *local_error;
-        DBusGConnection *connection;
-        gboolean         res;
-
-        g_debug ("GdmGreeterLoginWindow: system stop auth callback gained=%s", gained_privilege ? "yes" : "no");
-
-        if (! gained_privilege) {
-                if (error != NULL) {
-                        g_warning ("GdmGreeterLoginWindow: system stop error: %s", error->message);
-                }
-                return;
-        }
-
-        local_error = NULL;
-        connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &local_error);
-        if (connection == NULL) {
-                g_warning ("Unable to get system bus connection: %s", local_error->message);
-                g_error_free (local_error);
-                return;
-        }
-
-        res = try_system_stop (connection, &local_error);
-        if (! res) {
-                g_warning ("Unable to stop system: %s", local_error->message);
-                g_error_free (local_error);
-                return;
-        }
-}
-
-static PolKitAction *
-get_action_from_error (GError *error)
-{
-        PolKitAction *action;
-        char         *paction;
-        char         *p;
-
-        action = polkit_action_new ();
-
-        paction = NULL;
-        if (g_str_has_prefix (error->message, "Not privileged for action: ")) {
-                paction = g_strdup (error->message + strlen ("Not privileged for action: "));
-                p = strchr (paction, ' ');
-                if (p != NULL) {
-                        *p = '\0';
-                }
-        }
-        g_debug ("GdmGreeterLoginWindow: Requesting priv for '%s'", paction);
-
-        polkit_action_set_action_id (action, paction);
-
-        g_free (paction);
-
-        return action;
-}
-
-static PolKitResult
-get_result_from_error (GError *error)
-{
-        PolKitResult result = POLKIT_RESULT_UNKNOWN;
-        const char  *p;
-
-        p = strrchr (error->message, ' ');
-        if (p != NULL) {
-                p++;
-                polkit_result_from_string_representation (p, &result);
-        }
-
-        return result;
-}
-#endif
-
 static void
 do_system_restart (GdmGreeterLoginWindow *login_window)
 {
@@ -1163,69 +1086,12 @@ do_system_restart (GdmGreeterLoginWindow *login_window)
         }
 
         res = try_system_restart (connection, &error);
-#ifdef HAVE_POLKIT_GNOME
-        if (! res) {
+        if (!res) {
                 g_debug ("GdmGreeterLoginWindow: unable to restart system: %s: %s",
                          dbus_g_error_get_name (error),
                          error->message);
-
-                if (dbus_g_error_has_name (error, "org.freedesktop.ConsoleKit.Manager.NotPrivileged")) {
-                        PolKitAction *action;
-                        PolKitAction *action2;
-                        PolKitResult  result;
-                        GtkWidget    *dialog;
-                        guint         xid;
-                        pid_t         pid;
-
-                        result = get_result_from_error (error);
-                        action = get_action_from_error (error);
-
-                        if (result == POLKIT_RESULT_NO) {
-                                action2 = polkit_action_new ();
-                                polkit_action_set_action_id (action2,
-                                                             "org.freedesktop.consolekit.system.restart-multiple-users");
-                                dialog = gtk_message_dialog_new (GTK_WINDOW (login_window),
-                                                                 GTK_DIALOG_MODAL,
-                                                                 GTK_MESSAGE_ERROR,
-                                                                 GTK_BUTTONS_OK,
-                                                                 _("Failed to restart computer"));
-                                if (polkit_action_equal (action, action2)) {
-                                        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                                                  _("You are not allowed to restart the computer "
-                                                                                    "because multiple users are logged in"));
-                                }
-                                gtk_dialog_run (GTK_DIALOG (dialog));
-                                gtk_widget_destroy (dialog);
-
-                                polkit_action_unref (action);
-                                polkit_action_unref (action2);
-
-                                g_error_free (error);
-
-                                return;
-                        }
-
-                        xid = 0;
-                        pid = getpid ();
-
-                        g_error_free (error);
-                        error = NULL;
-                        res = polkit_gnome_auth_obtain (action,
-                                                        xid,
-                                                        pid,
-                                                        (PolKitGnomeAuthCB) system_restart_auth_cb,
-                                                        login_window,
-                                                        &error);
-                        polkit_action_unref (action);
-
-                        if (! res) {
-                                g_warning ("Unable to request privilege for action: %s", error->message);
-                                g_error_free (error);
-                        }
-
-                }
+		g_error_free (error);
         }
-#endif
 }
 
 static void
@@ -1244,67 +1110,12 @@ do_system_stop (GdmGreeterLoginWindow *login_window)
         }
 
         res = try_system_stop (connection, &error);
-#ifdef HAVE_POLKIT_GNOME
-        if (! res) {
+        if (!res) {
                 g_debug ("GdmGreeterLoginWindow: unable to stop system: %s: %s",
                          dbus_g_error_get_name (error),
                          error->message);
-
-                if (dbus_g_error_has_name (error, "org.freedesktop.ConsoleKit.Manager.NotPrivileged")) {
-                        PolKitAction *action;
-                        PolKitAction *action2;
-                        PolKitResult  result;
-                        GtkWidget    *dialog;
-                        guint         xid;
-                        pid_t         pid;
-
-                        xid = 0;
-                        pid = getpid ();
-
-                        result = get_result_from_error (error);
-                        action = get_action_from_error (error);
-
-                        if (result == POLKIT_RESULT_NO) {
-                                action2 = polkit_action_new ();
-                                polkit_action_set_action_id (action2,
-                                                             "org.freedesktop.consolekit.system.stop-multiple-users");
-                                dialog = gtk_message_dialog_new (GTK_WINDOW (login_window),
-                                                                 GTK_DIALOG_MODAL,
-                                                                 GTK_MESSAGE_ERROR,
-                                                                 GTK_BUTTONS_OK,
-                                                                 _("Failed to stop computer"));
-                                if (polkit_action_equal (action, action2)) {
-                                        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                                                  _("You are not allowed to stop the computer "
-                                                                                    "because multiple users are logged in"));
-                                }
-                                gtk_dialog_run (GTK_DIALOG (dialog));
-                                gtk_widget_destroy (dialog);
-
-                                polkit_action_unref (action);
-                                polkit_action_unref (action2);
-
-                                return;
-                        }
-
-                        g_error_free (error);
-                        error = NULL;
-                        res = polkit_gnome_auth_obtain (action,
-                                                        xid,
-                                                        pid,
-                                                        (PolKitGnomeAuthCB) system_stop_auth_cb,
-                                                        login_window,
-                                                        &error);
-                        polkit_action_unref (action);
-
-                        if (! res) {
-                                g_warning ("Unable to request privilege for action: %s", error->message);
-                                g_error_free (error);
-                        }
-
-                }
+                g_error_free (error);
         }
-#endif
 }
 
 static void
@@ -1323,10 +1134,22 @@ shutdown_button_clicked (GtkButton             *button,
         do_system_stop (login_window);
 }
 
+static void
+on_user_chooser_visibility_changed (GdmGreeterLoginWindow *login_window)
+{
+        update_banner_message (login_window);
+        adjust_other_login_visibility (login_window);
+}
+
 static void
 on_users_loaded (GdmUserChooserWidget  *user_chooser,
                  GdmGreeterLoginWindow *login_window)
 {
+        g_debug ("GdmGreeterLoginWindow: users loaded");
+        login_window->priv->user_chooser_loaded = TRUE;
+        update_banner_message (login_window);
+        adjust_other_login_visibility (login_window);
+
         gdm_chooser_widget_activate_if_one_item (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
 }
 
@@ -1598,6 +1421,11 @@ load_theme (GdmGreeterLoginWindow *login_window)
                           G_CALLBACK (on_user_unchosen),
                           login_window);
 
+        g_signal_connect_swapped (login_window->priv->user_chooser,
+                                 "notify::list-visible",
+                                 G_CALLBACK (on_user_chooser_visibility_changed),
+                                 login_window);
+
         gtk_widget_show (login_window->priv->user_chooser);
 
         login_window->priv->auth_banner_label = glade_xml_get_widget (login_window->priv->xml, "auth-banner-label");
@@ -1656,16 +1484,22 @@ static void
 gdm_greeter_login_window_size_request (GtkWidget      *widget,
                                        GtkRequisition *requisition)
 {
-        int            screen_w;
-        int            screen_h;
-        GtkRequisition child_requisition;
+        int             monitor;
+        GdkScreen      *screen;
+        GtkRequisition  child_requisition;
+        GdkRectangle    area;
 
         if (GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_request) {
                 GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_request (widget, requisition);
         }
 
-        screen_w = gdk_screen_get_width (gtk_widget_get_screen (widget));
-        screen_h = gdk_screen_get_height (gtk_widget_get_screen (widget));
+        if (!GTK_WIDGET_REALIZED (widget)) {
+                return;
+        }
+
+        screen = gtk_widget_get_screen (widget);
+        monitor = gdk_screen_get_monitor_at_window (screen, widget->window);
+        gdk_screen_get_monitor_geometry (screen, monitor, &area);
 
         gtk_widget_get_child_requisition (GTK_BIN (widget)->child, &child_requisition);
         *requisition = child_requisition;
@@ -1676,8 +1510,12 @@ gdm_greeter_login_window_size_request (GtkWidget      *widget,
         /* Make width be at least 33% screen width
          * and height be at most 80% of screen height
          */
-        requisition->width = MAX (requisition->width, .33 * screen_w);
-        requisition->height = MIN (requisition->height, .80 * screen_h);
+        requisition->width = MAX (requisition->width, .33 * area.width);
+        requisition->height = MIN (requisition->height, .80 * area.height);
+
+       /* Don't ever shrink window width
+        */
+       requisition->width = MAX (requisition->width, widget->allocation.width);
 }
 
 static void
@@ -1705,9 +1543,23 @@ update_banner_message (GdmGreeterLoginWindow *login_window)
                 g_debug ("GdmGreeterLoginWindow: banner message disabled");
                 gtk_widget_hide (login_window->priv->auth_banner_label);
         } else {
-                char *message;
+                char *message = NULL;
                 error = NULL;
-                message = gconf_client_get_string (login_window->priv->client, KEY_BANNER_MESSAGE_TEXT, &error);
+                if (login_window->priv->user_chooser_loaded && gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) == 0) {
+                        message = gconf_client_get_string (login_window->priv->client, KEY_BANNER_MESSAGE_TEXT_NOCHOOSER, &error);
+                        if (error != NULL) {
+                                g_debug("GdmGreeterLoginWindow: unable to get nochooser banner text: %s", error->message);
+                                g_error_free(error);
+                        }
+                }
+                error = NULL;
+                if (message == NULL) {
+                        message = gconf_client_get_string (login_window->priv->client, KEY_BANNER_MESSAGE_TEXT, &error);
+                        if (error != NULL) {
+                                g_debug("GdmGreeterLoginWindow: unable to get banner text: %s", error->message);
+                                g_error_free(error);
+                        }
+                }
                 if (message != NULL) {
                         char *markup;
                         markup = g_markup_printf_escaped ("<small><i>%s</i></small>", message);
@@ -1879,7 +1731,7 @@ on_gconf_key_changed (GConfClient           *client,
                         g_warning ("Error retrieving configuration key '%s': Invalid type",
                                    key);
                 }
-        } else if (strcmp (key, KEY_BANNER_MESSAGE_TEXT) == 0) {
+        } else if (strcmp (key, KEY_BANNER_MESSAGE_TEXT) == 0 || strcmp (key, KEY_BANNER_MESSAGE_TEXT_NOCHOOSER) == 0) {
                 if (login_window->priv->banner_message_enabled) {
                         update_banner_message (login_window);
                 }
@@ -1921,6 +1773,7 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
         gtk_window_set_skip_taskbar_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_set_skip_pager_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_stick (GTK_WINDOW (login_window));
+        gtk_window_set_type_hint (GTK_WINDOW (login_window), GDK_WINDOW_TYPE_HINT_DOCK);
         gtk_container_set_border_width (GTK_CONTAINER (login_window), 0);
 
         g_signal_connect (login_window,
